;------------------------------------------------------------------------------
; TINY ES1988 EPROM-Bank
; (c) V. Pohlers 2021
;------------------------------------------------------------------------------
; Bank-Steuersoftware, Bank 0
; wird im ES 1988X mit Mon X gestartet
; 08.10.2021, letzte nderung 
;------------------------------------------------------------------------------


;AS-Funktionen
hi              function x,(x>>8)&255
lo              function x, x&255

;------------------------------------------------------------------------------

		cpu	z8601


P3		EQU	03H		; Port 3
FLAGS		EQU	0FCH		; Flagregister
RP		EQU	0FDH		; Registerpointer
SPH		EQU	0FEH		; Stackpointer, Highteil
SPL		EQU	0FFH		; Stackpointer, Lowteil

;ramprog	equ	0F400h		; wo ist das am besten aufgehoben ??? 100h Platz reicht
ramprog		equ	0FC00h		; 128 Byte Magnetbandpuffer 
					; gefolgt von 128 Byte frei fr Standard-Programme (etwa Grafikerw.)
					
; ROM-Bank Eigenschaften
bankstart	equ	4000h
blocksize	equ	2000h	; 8K
lastbank	equ	00fh	; 16 Bnke
systembank	equ	0

packed		set	0		; keine packed Programme zulassen
;------------------------------------------------------------------------------
; ES-Funktionen

		include	es1988.asm

WriteBlock	equ	274Eh
resbnk		equ	26B3h		; Bank 0 setzen
nextbnk		equ	26D6h		; Bankweiterschaltung
setbank		equ	26DDh		; setze Bank %58

HTA8		equ	20B6h		; Hexadezimalanzeige eines Bytes aus %5D, Kursor (%5B) wird aktualisiert

BASBegin	EQU 	008FFh		; Start BASIC-Modus

;------------------------------------------------------------------------------

		org	bankstart
		assume RP:0C0h			; keine Optimierung !

		; sprungverteiler
		jp	main			; Hauptprogramm, Start mit Mon X
		jp	3000h			; baserw
		jp	0			; Suchroutine umladen
		jp	0			; Lade+Entpackroutine umladen
		jp	0			; Routine starten
		jp	0			; Programm starten
		;--
		db	"VP4"			; Kennung fr Banksuche0
		jp	ES_SaveX		; aus ES1988 ausgelagert

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; aus ES1988 ausgelagert

ES_SaveX	ld  5Bh, #20h		; Cursor in dritte Zeile
ES_Save3	call TBS		; auf Tastendruck warten
		cp 5Ah, #0Dh		; Enter?
		jr NZ, ES_Save3		; nein, nchstes Zeichen

; der Programmname steht jetzt im Bildschirmspeicher ab %FD20
		di     			; Video aus
		ld R0, #0FCh		; Pufferadresse
		ld 03h, R0
		ld R1, #0
		ld R2, #0FDh		; Anfang des Programmnamens
		ld R3, #020h
		ld R4, #11
ES_Save4	lde R5, @RR2		; Zeichen holen
		lde @RR0, R5		; und in den Puffer schreiben
		inc R1
		inc R3
		djnz R4, ES_Save4

; restlichen Puffer lschen
ES_Save5	lde @RR0, R4		; Null
		inc R1
		jr pl, ES_Save5		; bis %FC80 auffllen

		ld R5, #2
		ld R1, #10h
		lde @RR0, R5		; %FC10 = 02
		inc R1
		ld R5, #50h		; Start- und Endadresse LSB.MSB (!)
		ld R4, #4
ES_Save6	ldei @RR0, @r5		;  einragen ab %FC11
		djnz R4, ES_Save6

		ld R1, #0		; Zeiger wieder auf Anfang des Puffers
		ld R6, #1		; Blocknummer 1
		ld R12, #32		; Lnge Vorton
		call WriteBlock		; wegschreiben
		ld R0, 51h		; Programmstart
		ld R1, 50h
		ld R2, 53h		; Programmende
		ld R3, 52h
ES_Save8	inc R6			; nchster Block
		ld R4, R2
		ld R5, R3
		sub R5, R1		; Restlnge bestimmen
		sbc R4, R0
		jr NC, ES_Save7		; noch was da --> weiter
		ret    			; sonst fertig

; nchster Block
ES_Save7	sub R5, #128
		sbc R4, R12
		jr NC, ES_Save9		; letzter Block?
		ld R6, #255		;  ja, Blocknummer 255
ES_Save9	ld R13, #160
		call WriteBlock		; sonst wegschreiben
		jr ES_Save8

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; Anzeige ROM-Bankinhalt
; Auswahl eines Programms und Start

main:		srp	#20h


		call TBS		; auf Tastendruck warten
		cp 5Ah, #'D'		; Dir?
		jr Z, maind		; nein, nchstes Zeichen
		
		; ansonsten Taste in Tastaturpuffer zurck und Nr.Eingabe
		ld TAST, 5Ah
		or TAST,#80h		; setze T-Bit
		ld 5Ah, #08h		; CLR
		call	ZBS
		jr	main2

		; DIR
maind:		call	dir0			; Umladen
		call	ramprog			; und DIR starten
		
		; Eingabe verarbeiten
main2:		call	numin
		cp	r10, #0
		jr 	nz, main1		; gltige Zahl
		;sonst Rcksprung in Monitor
mainret:	jp 	ES_Begin

main1:		call	search0			; Umladen 
		di     				; Video aus
		call	ramprog			; und search starten
		ei

; out: 2Ah = 0 -> nicht gefunden
; 	2Bh (r11) bank
;	2Ch (r12) Hi Adr

		cp	r10, #0
		jr	nz,main3
		; nicht gefunden
		ld	5Ah,#'?'
		;ld	5Ah,#'!'
    		call	ZBS			; Ausgabe Fehler
		jr	main2			; neue Eingabe

main3:		;load
		call	load0			; Umladen
		di     				; Video aus
		call	ramprog			; und Umladen starten
		ei

		; jetzt rp = 10h !!
		; in 6/7 steht die Startadr

		; starte Programm
start_prog:	cp	r10,#'M'
		jr	eq, start_mc

;		cp	r10,#'P'
;		jr	eq, start_bmp

		cp	r10,#'B'
		jr	nz,mainret	; sonstige Typen nur laden, kein start
		
		
spsav		equ	0FD80h	; unteres Ende Stack-Bereich
		
		; starte BASIC
start_basic:	jp	BASBegin
;;		; SP sichern
;;		ld	R0,#hi(spsav)
;;		ld	R1,#lo(spsav)
;;		ld	R2,#SPH
;;		ldei	@RR0,@R2
;;		ldei	@RR0,@R2
;;
;;		CLR 8 			; keine Proctab
;;		CLR 9			; in 6/7 = Adr Basic-Pgm (Std. E000)
;;		
;;		call  07FDh 		; Interpreter starten
;;		; kein jp basic, da Stackpointer durch gosub etc nicht passen muss
;;		
;;		; SP wiederherstellen
;;		di
;;		ld	R0,#hi(spsav)
;;		ld	R1,#lo(spsav)
;;		ld	R2,#SPH
;;		ldei	@R2,@RR0
;;		ldei	@R2,@RR0
;;		ei
		jr	mainret		

start_mc:	; starte MC

		cp	6,#0
		jr	eq, mainret	; kein Autostart
		jp	@6
		
		jr	mainret

;		; Starte Bildbetrachter (todo)
;
;start_bmp:	;CALL	PRISTRI
;		;db	"BMP-Viewer fehlt noch",0dh,0
;		;ret
;		jp	bmpview
;
;		include	"Software/Rolf/Bildbetrachter/bildbetrachter.asm"

;------------------------------------------------------------------------------
; Umladeroutinen
; whrend Aktionen mit der ROM-Bank ist dieser Codebereich nicht nutzbar,
; da diese Bank weggeschaltet wird, deshalb Umladen in RAM (-> ramprog) 

dir0:		ld	r0,#hi(diranf)
		ld	r1,#lo(diranf)
		ld	r4,#hi(dirlen)
		ld	r5,#lo(dirlen)
		jp	ldir			; Umladen und DIR starten

search0:	ld	r0,#hi(searchanf)
		ld	r1,#lo(searchanf)
		ld	r4,#hi(searchlen)
		ld	r5,#lo(searchlen)
		jp	ldir			; Umladen und search starten

load0:		ld	r0,#hi(loadanf)
		ld	r1,#lo(loadanf)
		ld	r4,#hi(loadlen)
		ld	r5,#lo(loadlen)
		jp	ldir			; Umladen und load starten


;------------------------------------------------------------------------------
; Zahleneneingabe
; ret	r10: nummer
numin:		srp	#10h
		call	2E9h		; Input vom BASIC
		srp	#20h
		ld	r10, 13h	; rr2 enthlt zahl
		ret

;------------------------------------------------------------------------------
; Bereich nach ramprog verschieben + starten
; in rr0 quelle, rr4 laenge

ldir:		ld	r2,#hi(ramprog)
		ld	r3,#lo(ramprog)
mov1:		lde	r6,@rr0
		lde	@rr2,r6
		incw	rr0
		incw	rr2
		decw	rr4
		jr	nz, mov1
		;jp	ramprog		; starten
		ret

;------------------------------------------------------------------------------
; Stringausgabe, String folgt nach CALL, Ende mit 0
; in rp=20h

pristri:	pop	20h
		pop	21h
pristri1:	ld	R2, #5Ah
		ldei	@R2, @RR0
		or	5Ah, 5Ah	; 0?
		jr	Z, pristri2
		call	ZBS
		jr	pristri1
pristri2:	jp	@20h

;------------------------------------------------------------------------------
; DIR
;------------------------------------------------------------------------------

diranf		equ	$

		phase	ramprog

dir:		srp	#20h

    		CALL	PRISTRI
    		db	0ch,"ROMBANK VP'21",0dh,"-------------",0Dh
    		db	0

		call	resbnk		;systembank	
		ld	58h, #0		;systembank
		ld	r13,#0		;lfd nr

		ld	r12,#0+2	;lfd nr Anzeige (2 Zeilen Titel lassen)
		ld	r14, #hi(bankstart)
dir3:		clr	r15		; xx00-Adresse
		lde	r5,@rr14
		cp	r5,#0fah
		jr	nz,dir2
		incw	rr14
		lde	r5,@rr14
		cp	r5,#0fah
		jr	nz,dir2
		; Kennung gefunden
		call	show

		cp	r12,#7		; max 7 Eintrge auf einmal anzeigen
		jr	lt,dir2

		call	TSK		;Pause, warten auf taste
		ld	r12,#0		;Zhler wieder auf 0 setzen

dir2:		;weitersuchen
		inc	r14
		cp	r14, #hi(bankstart+blocksize)
		jr	ult,dir3
		;next bank
		ld	r14,#hi(bankstart)	; zurck auf 2000h
		clr	r15
		call	nextbnk
		inc	58h
		cp	58h,#lastbank
		jr	ule, dir3

dir4:		; Ende
    		CALL	PRISTRI
    		db	0dh
    		DB	"NR? ",0

		;;;ld	r10,r14		;aktuelle Pos sichern
		;;;ld	r11,54h		;aktuelle Bank sichern

		call	resbnk		;systembank
		ld	58h, #0		;systembank
		ret

show:  		add	r13,#1		; lfd Nr erhhen
		da	r13		; dezimal
		inc	r12

		ld	5Dh,r13
    		call	HTA8		; lfd Nr anzeigen
		;
    		ld	5Ah,#' '
    		call	ZBS
    		ld	r15, #15	; offs. Typ
    		lde	r5,@rr14
    		ld	5Ah,r5
    		call	ZBS
    		ld	5Ah,#' '
    		call	ZBS

   		; Anzeige Programmname
    		ld	r1,#8		; Name 16 zeichen,  aber
    		; nur 8 anzeigen, dann passt das pro Zeile

    		ld	r15,#10h	; offs. Name
show1:    	lde	r5,@rr14
		ld	5Ah,r5
    		call	ZBS
    		incw	r14
   		djnz	r1, show1
		ret

		dephase

dirlen		equ	$-diranf

;------------------------------------------------------------------------------
; search
; in: 2Ah  (r10) gesuchte Nummer
; out: 2Ah = 0 -> nicht gefunden
; 	2Bh (r11) bank
;	2Ch (r12) Hi Adr
;------------------------------------------------------------------------------

searchanf	equ	$

		phase	ramprog

search:		srp	#20h
		ld	58h, #0
		ld	r13,#0		;lfd nr
		ld	r14, #hi(bankstart)
search3:	clr	r15		; xx00-Adresse
		lde	r5,@rr14
		cp	r5,#0fah
		jr	nz,search2
		incw	rr14
		lde	r5,@rr14
		cp	r5,#0fah
		jr	nz,search2
		; Kennung gefunden
		inc	r13
		cp	r13,r10		; vergleich mit gesuchter nummer
		jr	z, search4	; gefunden
search2:	;weitersuchen
		inc	r14
		cp	r14, #hi(bankstart+blocksize)
		jr	ult,search3
		;next bank
		ld	r14,#hi(bankstart)	; zurck auf 2000h
		clr	r15
		call	nextbnk
		inc	58h
		cp	58h,#lastbank
		jr	ule, search3
		ld	r10,#0		; alles durchsucht, aber nicht gefunden
search4:	; Ende
		ld	r11,58h		; aktuelle Bank
		ld	r12,r14		; akuelle Pos

		call	resbnk		;systembank
		ret

		dephase
searchlen	equ	$-searchanf

;------------------------------------------------------------------------------
; load
; 	2Bh (r11) bank
;	2Ch (r12) Hi Adr
; ret: 	RP = 10h (!)
; 	r10 = type
; 	6/7 = sadr
;------------------------------------------------------------------------------

loadanf		equ	$
		phase	ramprog

	srp 	#10h

	ld	58h, 2Bh	;#prgbank
	call	setbank
	ld	r4, 2Ch		;hi(prgpos), lo ist immer xx00h
;
	ld	r5,#6		; offset sadr
	lde	r10,@rr4
	incw	rr4
	lde	r11,@rr4
	push	r10		; save saddr
	push	r11

	ld	r5,#15		; offset type
	lde	r6,@rr4
	push	r6		; save type


	; Quelladresse rr0
	ld	r0,r4		; hi pos
	ld	r1,#20h		; lo pos ist immer xx00h + headersize = 20h

	; Zieladresse rr2
	ld	r5,#2		; offs. aadr
	lde	r2,@rr4		; hi aadr
	incw	rr4
	lde	r3,@rr4		; lo aadr

	ld	r5,#8		; offs. packed
	lde	r6,@rr4
	tm	r6,#80h
	if packed = 1
	jr	z, copy1
	; unpack
	call	depack
	jr	copy2
	endif

copy1:	; einfaches ldir
	ld	r5,#4		;offs. eadr
	lde	r6,@rr4
	incw	rr4
	lde	r7,@rr4
	sub	r7,r3
	sbc	r6,r2		; rr6 = laenge

copy1a:
	call	ldi		; copy from rr0 to rr2
	decw	rr6
	jr	nz, copy1a

copy2:	call	resbnk		; Bank 0

	pop	r10		; r10 = type
	pop	7
	pop	6		; 6 = sadr


	ret

	if packed = 1
	include	depack_z8b.asm
	else
;UP: transfer from rr0 to rr2
ldi:		lde	r5,@rr0			;get next byte
		incw	rr0			;increase source data address
		;ggf hier bankumschaltung
		cp	r0,#hi(bankstart+blocksize)	; endadresse erreicht?
		jr	ult,getbyte1	
		call	nextbnk			;next bank
		ld	r0,#hi(bankstart)	; zurck auf Anf
		clr	r1
getbyte1:	lde	@rr2,r5
		incw	rr2
		ret
	endif

		dephase
loadlen		equ	$-loadanf

;------------------------------------------------------------------------------

		end
