;------------------------------------------------------------------------------
; TINY ES40 EPROM-Bank
; (c) V. Pohlers 2021
;------------------------------------------------------------------------------
; Zusammenstellung Bankinhalt, erzeugt den ROM
; 21.08.2021, letzte nderung 
;------------------------------------------------------------------------------
; bei nderungen in dieser Datei unbedingt 'make depend' starten!
;------------------------------------------------------------------------------

;        cpu 96C141			; Mikrocontroller mit groem Adressbereich
;        maxmode on			; der Z8 reicht hier nicht.
					; leider little endian
	
	cpu 80C251			; geht mit bigendian :-)
        BIGENDIAN on

;	MACEXP  off

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

;AS-Funktionen
hi              function x,(x>>8)&255
lo              function x, x&255
x_align	function x, (($+x-1)/x)*x

myalign	macro	size
	org	x_align(size)
	endm

bankstart	equ	2000h
blocksize	equ	2000h	; 8K
lastbank	equ	00fh	; 
systembank	equ	0

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; MACRO a d d f i l e
;
	;addFile filename, aadr, [eadr], [sadr], typ, name, kommentar
;
;filename mit Pfad
;eadr kann bei ungepackten Daten leer bleiben (dann = aadr+filelength)
;sadr kann leer bleiben (dann = aadr)
;typ = Dateityp (B = Tiny-MPBASIC, M = MC-Binary)
;Name = xx Zeichen
;Kommentar = xx 13 Zeichen
;
; align	100h verschwendet etwas Speicher, aber ist vermutlich leichter zu handlen
; fileXname als Dummy anlegen

addFile	macro	fileXname,aadr,eadr,sadr,typ,name

		;align	20h	
		align	100h	

bank 	eval 	($ / blocksize)
pos	eval 	($ # blocksize + bankstart)

	if	"eadr"<>""
eaddr	eval	eadr
	else
eaddr	eval	aadr+file_bis-file_von
	endif
	if	"sadr"<>""
saddr	eval	sadr
	else
saddr	eval	aadr
	endif

len 	eval	eaddr-aadr+1

packed	eval	0
	if STRSTR(fileXname,".pck") > 0
packed	eval	80h
eaddr 	eval	0
	endif
	
		; Header
		db	0FAH,0FAH			; 2 magic marker FlAsH
		dw	aadr, eaddr, saddr		; 6
		;;db	hi(aadr), lo(aadr)
		;;db	hi(eaddr), lo(eaddr)
		;;db	hi(saddr), lo(saddr)
		db	packed
		db	6 dup(0)			; reserviert
		db	typ				; 1
		db	upstring(substr(name+"                ",0,16))	; 16

		; file
		align	20h			; align knnte entfallen; Header ist 20h lang
file_von
		BINCLUDE  fileXname
file_bis
		MESSAGE "\{bank}, \{pos}h, aadr, \{eaddr}h, \{saddr}h, typ, "+name+", \{len}, "+fileXname

		endm

;------------------------------------------------------------------------------

	;bank0
	
	binclude	bank0.bin
	org	1000h			; offs 3000h
	binclude	"Software/volker/baserw2k.bin"

	; wg. Kompatibilitt zur orig. ROM-Bank
	; CALL %A4A -> CALL 37E0 laden Z1013-ZS
	;org	17D0h			; offs 37D0h
	;binclude	"zg.bin"

;------------------------------------------------------------------------------

	align	blocksize

	;addFile filename, aadr, [eadr], [sadr], typ, name
	;Packen - Endung pck ergnzen!

	;bei jtc -> include file.bin.inc
	;sonst addFile "Software/basic2k/autocross.bin", 0E000h, 0E31Fh, 000h, 'B', "Autocross" 

	include Software/basic2k/autocross.bin.inc		; JU+TE 6/1988, Seiten 442-445
	include Software/basic2k/autocross_plus.bin.inc		; erw. Tasten f. JTCEMU
	include Software/basic2k/ein_mal_eins.bin.inc		; JU+TE 7/1989, Seiten 550-553
	include Software/basic2k/grafiklib_fca0.bin.inc		; JU+TE 3/1988, Seiten 232-235
	include Software/basic2k/grafiktest.bin.inc		; JU+TE 3/1988, Seiten 232-235	braucht grafiklib_fca0
	include Software/basic2k/hase_wolf.bin.inc		; JU+TE 3/1988, Seiten 232-235
	include Software/basic2k/hase_wolf_plus.bin.inc		; erw. Tasten f. JTCEMU
	include Software/basic2k/kleines1x1.bin.inc		; JU+TE 7/1989, Seiten 550-553
	include Software/basic2k/kreisberechnung.bin.inc	; JU+TE 12/1987, Seiten 930-932
	include Software/basic2k/kurzzeitwecker.bin.inc		; JU+TE 4/1989, Seiten 310-313
	include Software/basic2k/landstrasse.bin.inc		; JU+TE 11/1989, Seiten 871-873
	include Software/basic2k/malfix.bin.inc			; JU+TE 1/1989, Seiten 70-73	braucht grafiklib_fca0
	include Software/basic2k/mastermind.bin.inc		; JU+TE 7/1989, Seiten 550-553
	include Software/basic2k/melodie.bin.inc		; JU+TE 4/1989, Seiten 310-313
	include Software/basic2k/mondlandung_v1.bin.inc		; JU+TE 12/1987, Seiten 930-932
	include Software/basic2k/mondlandung_v2.bin.inc		; JU+TE 3/1988, Seiten 232-235	braucht grafiklib_fca0
	include Software/basic2k/monophon.bin.inc		; JU+TE 1/1989, Seiten 70-73
	include Software/basic2k/pasch.bin.inc			; JU+TE 6/1989, Seiten 470-473
	include Software/basic2k/primzahlen.bin.inc		; JU+TE 05/1990, Seiten 386-389
	include Software/basic2k/quadratwurzel.bin.inc		; JU+TE 6/1988, Seiten 442-445
	include Software/basic2k/reaktionstest_v1.bin.inc	; JU+TE 4/1989, Seiten 310-313
	include Software/basic2k/reaktionstest_v2.bin.inc	; JU+TE 4/1989, Seiten 310-313
	include Software/basic2k/roemische_zahlen.bin.inc	; JU+TE 7/1989, Seiten 550-553
	include Software/basic2k/schiebepuzzle.bin.inc		; JU+TE 02/1990, Seiten 142-144
	include Software/basic2k/schiebepuzzle_plus.bin.inc	; erw. Tasten f. JTCEMU
	include Software/basic2k/solohalma.bin.inc		; JU+TE 02/1990, Seiten 142-144
	include Software/basic2k/turm_von_hanoi.bin.inc		; JU+TE 9/1989, Seiten 710-713
	include Software/basic2k/wochentag.bin.inc		; JU+TE 1/1989, Seiten 70-73
	include Software/basic2k/zahlenraten.bin.inc		; JU+TE 6/1988, Seiten 442-445
;
	include Software/jute/u2716-handler.bin.inc		; JU+TE 11/1988, Seiten 866-869
	include Software/jute/Lauflicht.bin.inc			; JU+TE 4/1988, Seiten 286-288
	include Software/jute/forth_4k.bin.inc			; JU+TE 01/1990, Seiten 70-73
	include Software/jute/oszi.bin.inc			; JU+TE 11/1989, Seiten 871-873
	include Software/jute/eprommer_4k.bin.inc		; JU+TE 8/1989, Seiten 626-629
	include Software/jute/rechner+mtb.bin.inc		; JU+TE 05/1990, Seiten 386-389	incl MTB
;aus ASM
	addFile "Software/sonstige/kopieren.bin"	, 0F800h, , , 'M', "KOPIEREN" 	; E.Mueller
	addFile "Software/sonstige/t2verb-as.bin"	, 0F800h, , , 'M', "TOR2" 	; E.Mueller
;aus JTC, aber hier explizit wg. anderem Namen
	addFile "Software/sonstige/SPACE4.bin"		, 0E000h, , , 'M', "SPACE" 	; TH. RACKWITZ 
	include Software/sonstige/othello.bin.inc	; ???
	include Software/sonstige/Steine.bin.inc	; s. JU+TE 01/1990, Seiten 70-73
	include Software/sonstige/lauf_PW.bin.inc	; Peter Winges
; aus jute abgetippt, ungetestet	
	include Software/jute/s3004.bin.inc			; JU+TE 5/1989, Seiten 376-380	incl MTB
	include Software/jute/c520d_mess.bin.inc		; JU+TE 12/1989, Seiten 940-943
	include Software/jute/c520d_oszi.bin.inc		; JU+TE 12/1989, Seiten 940-943
;
	include Software/sonstige/ballern.bin.inc		; Wolfgang Kott
	include Software/sonstige/bandit.bin.inc		; Wolfgang Kott
	
;------------------------------------------------------------------------------

	if	$>=(lastbank+1)*blocksize
		WARNING "ROM-Kapazitt berschritten!"
	endif


	org	(lastbank+1)*blocksize -1
	db	42
        
        end
